# Dodona DrRacket Plugin

The Dodona DrRacket plugin allows students to submit [Dodona](https://dodona.ugent.be/) Exercises from the [DrRacket](https://racket-lang.org/) IDE.

## Installation instructions for students

Students can install the plugin through the DrRaket GUI as follows:

- Open DrRacket
- `File > Install Package`
- Paste the following url in the "Package Source" field: https://soft.vub.ac.be/SCPI/dodona-drracket-plugin.zip
- Press install and wait
- Restart DrRacket

Afterwards, the package can be updated through the "Currently Installed" tab of the DrRacket Package Manager (`File > Package Manager`) when needed.

In order to publish the plugin for students, the [following steps](https://docs.racket-lang.org/pkg/getting-started.html#%28part._manual-deploy%29) should be used:

- Ensure `plugin/user-api-token.txt` is empty or not present (unless you want students to submit using your credentials)
- `cd` to the parent directory of this repository
- `raco pkg create dodona-drracket-plugin`
- Move the created zip and checksum files to the scpi folder on the SOFT server

## Development setup

In order to test the package locally, the package should be installed with the [raco](https://docs.racket-lang.org/raco/) tool:

- `git clone git@gitlab.soft.vub.ac.be:Structuur1/dodona-drracket-plugin.git`
- `cd dodona-drracket-plugin`
- `raco pkg install --auto`

Additionally, you should set up a dodona test environment to test the plugin.
Instructions to do this can be found on the [dodona github page](https://github.com/dodona-edu/dodona).
Dodona is rather finnicky to set up, however, so it is recommend to reach out to somebody who has done this before.

# Implementation

The project consists of two main modules:

1. `dodona-api`: which allows Racket programmers to communicate with the Dodona server.
2. `plugin`, the plugin itself. It depends on `dodona-api`.

### `dodona-api-rkt`

The main entry point of `dodona-api` is the `dodona-client%` class, which communicates with the dodona server over HTTP(S).
Dodona replies are enconded in json, which is converted to a racket `hash`.

**dodona-api-rkt** represents Dodona resources in Racket. For example, a class `course%` represents a Dodona course in Racket.

**dodona-api-rkt** implements resource managers. For example, to manage courses (for example retrieve a course), a `course-manager%` has been implemented.

### `plugin`

The main file implementing the plugin is `plugin/main.rkt`, which uses `dodona-client%`.
As with all DrRacket plugins, the plugin is essentially a Racket mixin.
This mixin is used by DrRacket to add the plugin-defined behavior into the base class implementing the DrRacket Frame.

A small mixin that adds plugin-defined behavior to the base class implementing the DrRacket definitions window has also been implemented.

The implementation of the plugin consists of 8 main parts:

1. Dodona menu on top of the DrRacket Frame.
2. API-token menu item
3. Hierarchical menu to select a course, and a series
4. `on-dodona-connect` and `on-dodona-disconnect` methods
5. Helper functions to manage buttons
6. Submission buttons. Four are implemented, but only one is shown at a time.
7. Last submission button, that allows user to access his last submission
8. Initialization of the client, using a saved API token
