#lang racket
(require (prefix-in http: net/http-easy))
(require json)
(require "resources/user.rkt")
(require "responses/root-response.rkt")
(require "managers/user-manager.rkt")
(require "managers/course-manager.rkt")
(require "managers/series-manager.rkt")
(require "managers/exercise-manager.rkt")
(require "managers/submission-manager.rkt")

(provide dodona-client%)

; ----------------------------------------------------------------------------------------------------------
;    dodona-client% communicates through HTTP with the dodona server.
;    So, a dodona-client% can send GET and POST requests.
;    These requests are initiated by the resource managers, such as the course-manager%
;    E.g. if the course-manager% wants to get a course, it asks dodona-client% to perform a GET request
; ----------------------------------------------------------------------------------------------------------

(define dodona-client%
  (class object%
    (init-field host token)
    (super-new)

    (field [user-manager (new user-manager%     ;use built-in procedure get-field to access fields
                              [dodona-host host]
                              [dodona-client this])]
           [course-manager (new course-manager%
                                [dodona-host host]
                                [dodona-client this])]
           [series-manager (new series-manager%
                                [dodona-host host]
                                [dodona-client this])]
           [exercise-manager (new exercise-manager%
                                  [dodona-host host]
                                  [dodona-client this])]
           [submission-manager (new submission-manager%
                                    [dodona-host host]
                                    [dodona-client this])]
           [id #f])

    (define/private (request-header) ;header used for HTTP requests
      (hash 'Accept "application/json"
            'Authorization token
            'Content-Type "application/json"))

    ;;GET REQUEST
    (define/public (get url)
      (let ((tweaked-url (string-replace url
                                         "http://dodona.localhost:3000" ;with a local connection, http://127.0.0.1:3000 should be used instead
                                         host)))
        (http:response-json ;as json
         (http:get tweaked-url
                   #:headers (request-header)))))

    (define/public (get-root-response)
      ;send a get request to host url. Recall that header includes API token,
      ;allowing dodona to return a root-response which includes the user
      (new root-response% [root-response-data (get host)]))

    ;;POST REQUEST
    (define/public (post url submission-assoc-list)
      (http:response-json
       (http:post url
                  #:headers (request-header)
                  #:json (hasheq 'submission                         ;submission should have this format
                                 (make-hasheq submission-assoc-list))
                  )))))
