#lang racket
(require "abstract-manager.rkt")
(require "../resources/course.rkt")
(provide course-manager%)

;---------------------------------------------------------------------------------------------------
;course-manager% communicates with dodona-client% to manage courses on the dodona server.
;E.g. fetch a course (which is a resource) from the server.
;---------------------------------------------------------------------------------------------------

(define course-manager%
  (class abstract-manager%
    (init dodona-host dodona-client)
    (super-new [host dodona-host]
               [client dodona-client])

    (inherit url)
    (inherit-field client)
    (inherit-field host)

    (define endpoint-courses "/courses/~a") ;call format with it

    (define/public (get-course id-or-submission)
      (define (get-course-by-id id)
        (let ((course-data (send client get (url (format endpoint-courses id-or-submission)))))
            (new course% [course-data course-data])))

      (define (get-course-by-submission submission)
        (let ((course-url (send id-or-submission get-course-url)))
            (if (equal? course-url 'null)
                #f
                (let ((course-data (send client get course-url)))
                  (new course% [course-data course-data])))))

      (if (number? id-or-submission) 
          (get-course-by-id id-or-submission)
          (get-course-by-submission id-or-submission)))))
