#lang racket
(require "abstract-manager.rkt")
(require "../resources/exercise.rkt")
(provide exercise-manager%)

;---------------------------------------------------------------------------------------------------
;exercise-manager% communicates with dodona-client% to manage exercises on the dodona server.
;E.g. fetch an exercise (which is a resource) from the server.
;---------------------------------------------------------------------------------------------------

(define exercise-manager%
  (class abstract-manager%
    (init dodona-host dodona-client)
    (super-new [host dodona-host]
               [client dodona-client])

    (inherit url)
    (inherit get-all-pages-data)
    (inherit-field client)
    (inherit-field host)

    (define endpoint-activities "/activities")
    (define endpoint-activities-id (string-append
                                    endpoint-activities
                                    "/~a"))

    (define endpoint-courses-activities-id (string-append "/courses/~a"
                                                          endpoint-activities-id))

    (define/public (get-all series)
      ;get all the exercises of a given series
      (let* ((exercises-url (send series get-exercises-url))
             (exercises-data (filter (lambda (data)
                                       (equal? (hash-ref data 'type) "Exercise"))
                                     (let ((exercises (get-all-pages-data exercises-url)))
                                       exercises))))
        (map (lambda (exercise-data)
               (new exercise% [exercise-data exercise-data]))
             exercises-data)))

    (define/public (get-course-exercise course-id exercise-id)
      ;gets a specific exercise of a course 
      (new exercise% [exercise-data (send client get (url (format endpoint-courses-activities-id
                                                                  course-id
                                                                  exercise-id)))]))

    (define/public (get-exercise exercise-id)
      ;gets a specific exercise
      (new exercise% [exercise-data (send client get (url (format endpoint-activities-id
                                                                  exercise-id)))]))

    (define/public (get-submission-exercise submission)
      ;gets the exercise for a submission
      (let ((exercise-url (send submission get-exercise-url)))
        (new exercise% [exercise-data (send client get exercise-url)])))))
