#lang racket
(require "abstract-manager.rkt")
(require "../resources/submission.rkt")
(require "../resources/submission-info.rkt")
(require "../responses/submission-created-response.rkt")
(provide submission-manager%)

;---------------------------------------------------------------------------------------------------
;submission-manager% communicates with dodona-client% to manage submissions on the dodona server.
;E.g. submit an exercise to the server.
;---------------------------------------------------------------------------------------------------

(define submission-manager%
  (class abstract-manager%
    (init-field dodona-host dodona-client)
    (super-new [host dodona-host]
               [client dodona-client])

    (inherit url)
    (inherit-field client)
    (inherit-field host)

    (define user (send (send client get-root-response) get-user))

    (define endpoint-course-id-exercise-id "%s?course_id=~a&activity_id=~a") ;call format with it
    (define endpoint-exercise-id "?activity_id=~a")
    (define endpoint-submissions "/submissions")
    (define endpoint-submissions-id "/submissions/~a")

    (define/public (submit courseID seriesID exerciseID code-string)
      ;submit the code for a specific course, series and exercise
      (let ((assoc-lst (list (cons 'course_id courseID)
                             (cons 'series_id seriesID)
                             (cons 'exercise_id exerciseID)
                             (cons 'code code-string))))
        (new submission-created-response%
             [submission-created-data (send client post (url endpoint-submissions) assoc-lst)])))

    (define/public (submit-exercise exerciseID code-string)
      ;submit the code for a specific exercise
      (let ((assoc-lst (list (cons 'exercise_id exerciseID)
                             (cons 'code code-string))))
        (new submission-created-response%
             [submission-created-data (send client post (url endpoint-submissions) assoc-lst)])))

    (define (fetch-submission-from-url url)
      (new submission%
           [submission-data
            (send client get url)]))

    (define/public (get submission-id) ;gets a specific submission
      (fetch-submission-from-url (url (format endpoint-submissions-id submission-id))))

    (define/public (get-user-submissions-info) ;gets all submission-infos of the user
      (map (lambda (submission-info-data)
             (new submission-info% [submission-info-data submission-info-data]))
           (send client get (send user get-submissions-url))))

    (define/public (get-exercise-submissions-info exercise) ;gets all submission-infos of the user, for a given exercise
      (let ((exercise-id (send exercise get-id))) 
        (map (lambda (submission-info-data)
               (new submission-info% [submission-info-data submission-info-data]))
             (send client get (string-append (send user get-submissions-url)
                                             (format endpoint-exercise-id
                                                     exercise-id))))))))
