#lang racket
(provide (all-defined-out))
(require json)

;---------------------------------------------------------------------------------------------------
;  submission% represents a submission (resource) on the Dodona server.
;  So, data associated with a submission is encapsulated
;---------------------------------------------------------------------------------------------------

(define submission%
  (class object%
    (init-field submission-data)
    (super-new)

    (define/private (lookup key)
      (hash-ref submission-data key))

    (define/public (get-accepted?)
      (lookup 'accepted))

    (define/public (get-code)
      (lookup 'code))

    (define/public (get-course-url)
      (lookup 'course))

    (define/public (get-creation-timestamp)
      (lookup 'created_at))

    (define/public (get-exercise-url)
      (lookup 'exercise))

    (define/public (get-id)
      (lookup 'id))

    (define/public (get-status)
      (lookup 'status))

    (define/public (get-summary)
      (lookup 'summary))

    (define/public (get-url)
      (lookup 'url))     

    (define/public (get-submission-test)
      (new submission-test% [submission-test-data (with-input-from-string
                                                      (lookup 'result)
                                                    (λ () (read-json)))]))
    ))

(define submission-test%
  (class object%
    (init-field submission-test-data)
    (super-new)

    (define (lookup key)
      (hash-ref submission-test-data key))

    (define/public (get-accepted?)
      (lookup 'accepted))

    (define/public (get-description)
      (lookup 'description))

    (define/public (get-tests-description)
      (hash-ref (car (lookup 'groups)) 'description))

    (define/public (get-test-tabs)
      (map (lambda (hash)
             (new test-tab% [test-tab-data hash]))
           (lookup 'groups)))
    ))

(define test-tab%
  (class object%
    (init-field test-tab-data)
    (super-new)

    (define (lookup key)
      (hash-ref test-tab-data key))

    (define/public (get-badge-count)
      (lookup 'badgeCount))

    (define/public (get-description)
      (lookup 'description))

    (define/public (get-scheme-error)
      (hash-ref (car (hash-ref test-tab-data 'messages)) 'description))

    (define/public (get-test-groups)
      (map (lambda (hash)
             (new test-group% [test-group-data hash]))
           (lookup 'groups)))))

(define test-group% ;a group of cohesive test-cases
  (class object%
    (init-field test-group-data)
    (super-new)

    (define (lookup key)
      (hash-ref test-group-data key))
    
    (define/public (get-accepted?)
      (lookup 'accepted))

    (define/public (get-description)
      (lookup 'description))

    (define/public (get-test-cases)
      (map (lambda (hash)
             (new test-case% [test-case-data hash]))
           (lookup 'groups)))))

(define test-case%
  (class object%
    (init-field test-case-data)
    (super-new)
    
    (define (lookup key)
      (hash-ref test-case-data key))

    (define/public (get-description)
      (lookup 'description))

    (define/public (get-accepted)
      (lookup 'accepted))

    (define/public (get-test-case-tests)
      (map (lambda (hash)
             (new test-case-tests% [test-case-tests-data hash]))
           (lookup 'tests)))))

(define test-case-tests%
  (class object%
    (init-field test-case-tests-data)
    (super-new)

    (define (lookup key)
      (hash-ref test-case-tests-data key))

    (define/public (get-accepted)
      (lookup 'accepted))

    (define/public (get-expected)
      (lookup 'expected))

    (define/public (get-generated)
      (lookup 'generated))

    (define/public (get-test-code)
      (hash-ref (lookup 'description) 'description))))

