#lang racket
(require racket/gui/base)
(provide (all-defined-out))
(require "dodona-constants.rkt")

; --------------------------------------------------------------------------------------------
;    1: Geometry management
; --------------------------------------------------------------------------------------------
(define EXERCISE-CHOICE-MIN-WIDTH 150) ;min width for choices UI element
(define MAX-FRAME-WIDTH 1200)
(define MAX-FRAME-HEIGHT 750)

(define DEFAULT-BORDER 7)
(define DEFAULT-SPACING 7)
(define MINIMAL-BORDER 4)
(define MINIMAL-SPACING 4)
(define API-TOKEN-TEXT-FIELD-MIN-WIDTH 500)

; --------------------------------------------------------------------------------------------
;    2: Directories & Paths
; --------------------------------------------------------------------------------------------
(define IMAGES-DIR (build-path PLUGIN-DIR "images"))

; --------------------------------------------------------------------------------------------
;    3: Bitmaps
; --------------------------------------------------------------------------------------------
(define QUESTION-MARK-BITMAP (read-bitmap (build-path IMAGES-DIR "question-mark.png")))
(define CORRECT-CHECK-MARK-BITMAP (read-bitmap (build-path IMAGES-DIR "correct-check-mark.png")))
(define LARGE-CORRECT-CHECK-MARK-BITMAP (read-bitmap (build-path IMAGES-DIR "large-correct-check-mark.png")))
(define WRONG-CHECK-MARK-BITMAP (read-bitmap (build-path IMAGES-DIR "wrong-check-mark.png")))
(define LARGE-WRONG-CHECK-MARK-BITMAP (read-bitmap (build-path IMAGES-DIR "large-wrong-check-mark.png")))
(define SUBMIT-BITMAP (read-bitmap (build-path IMAGES-DIR "submit.png")))
(define SUBMITTING-BITMAP (read-bitmap (build-path IMAGES-DIR "submitting.png")))

; --------------------------------------------------------------------------------------------
;    4: Labels
; --------------------------------------------------------------------------------------------

;button labels
(define OK-BUTTON-LABEL "Ok")
(define SUBMIT-BUTTON-LABEL "Submit")
(define LOG-IN-BUTTON-LABEL "Log In")
(define GO-TO-DODONA-BUTTON-LABEL "Go to Dodona")
(define CANNOT-SUBMIT-BUTTON-LABEL "Cannot Submit")
(define SUBMITTING-BUTTON-LABEL "Submitting...")
(define COPY-TO-REPL-BUTTON-LABEL "Copy to REPL")
(define OPEN-IN-DODONA-BUTTON-LABEL "Open in Dodona")
(define LAST-SUBMISSION-BUTTON-LABEL "Last Submission")

;msg labels
(define NEW-API-TOKEN-VALID-MSG-LABEL "new API token is valid")
(define NEW-API-TOKEN-NOT-VALID-MSG-LABEL "new API token is NOT valid")
(define COMMUNICATING-WITH-DODONA-MSG-LABEL "Communicating with Dodona")
(define NO-VALID-API-TOKEN-MSG-LABEL "Please set a valid API token.\n\nOn the menu bar, navigate to Dodona/API Token")
(define NO-CONNECTION-MSG-LABEL "Could not connect to Dodona.\nPlease make sure you're connected to the internet")
(define NO-SELECTED-SERIES-MSG-LABEL "Please select a WPO.\n\nOn the menu bar, navigate to Dodona/Select WPO")
(define TESTING-CODE-MSG-LABEL "Dodona is testing your code.\n Please wait")
(define SUBMISSION-MSG-PREFIX-LABEL "Submission is ")
(define NULL-MSG-LABEL "None")
(define LEGENDE-PREFIX-MSG-LABEL "shown for each test case: ")
(define TEST-CODE-MSG-LABEL "tests")
(define EXPECTED-RESULT-MSG-LABEL "expected-result")
(define GENERATED-RESULT-MSG-LABEL "generated-result")
(define CORRECT?-MSG-LABEL "correct?")
(define COULD-CONNECT-MSG-LABEL "You're online. You can connect to Dodona.")
(define COULD-NOT-CONNECT-MSG-LABEL "You're offline. You cannot connect to Dodona.\nPlease make sure you're connected to the internet.")
(define INSTRUCTION-STEP1-MSG-LABEL "Step 1) Sign in on Dodona: ")
(define INSTRUCTION-STEP2-MSG-LABEL "Step 2) Once signed in, on your profile page, make a new API token")
(define INSTRUCTION-STEP3-MSG-LABEL "Step 3) Copy the API token to the textfield (above), and press Log In")
(define GENERAL-ERROR-TITLE-LABEL "Something went wrong.")
(define GENERAL-ERROR-LABEL "Something went wrong. Please inform us about this error so we can fix this error.")

;frame labels
(define NO-CONNECTION-FRAME-LABEL "Cannot connect to Dodona")
(define NO-VALID-API-TOKEN-FRAME-LABEL "You need a valid API token before you can submit")
(define NO-SELECTED-SERIES-FRAME-LABEL "No selected WPO")
(define SUBMISSION-FRAME-LABEL "Submission Report")
(define CONNECTION-STATUS-FRAME-LABEL "Your Dodona connection")

;menu/menu-item labels
(define DODONA-MENU-LABEL "Dodona")
(define CONNECTION-STATUS-ITEM-LABEL "Check your Dodona connection")
(define API-TOKEN-MENU-ITEM-LABEL "API Token")
(define SELECT-SERIES-MENU-ITEM-LABEL "Select WPO")

;text-field labels
(define API-TOKEN-TEXT-FIELD-LABEL "Your API Token: ")

;group-box labels
(define EXECUTED-TESTS-GROUP-BOX-LABEL "Executed Tests")
(define INSTRUCTIONS-GROUP-BOX-LABEL "Instructions")

; --------------------------------------------------------------------------------------------
;    5: Fonts
; --------------------------------------------------------------------------------------------
(define CODE-FONT (make-object font% 11.0 'modern)) ;14.0 is a good, readable size
(define BIG-FONT (make-object font% 12.0 'modern 'normal 'bold))
(define BOLD-FONT (make-object font% 12.0 'swiss 'normal 'bold))
(define SMALL-FONT (make-object font% 12.0 'swiss))