#lang racket
; --------------------------------------------------------------------------------------------
;                                      DrRacket Dodona Plugin
;
;                          Original implementation: Turgut Reis Kursun
;                      Software Languages Lab - Vrije Universiteit Brussel
; --------------------------------------------------------------------------------------------
(require drracket/tool mrlib/switchable-button racket/gui/base net/sendurl "gui-table.rkt")
(require "../dodona-api/dodona-client.rkt")
(require "constants/dodona-constants.rkt")
(require "constants/UI-constants.rkt")

(provide tool@)

(define tool@
  (unit
    (import drracket:tool^)
    (export drracket:tool-exports^)

    (define frame-mixin
      (mixin (drracket:unit:frame<%> (class->interface drracket:unit:frame%)) ()
        (super-new)
        (inherit get-definitions-text
                 get-button-panel
                 register-toolbar-button
                 register-toolbar-buttons
                 sort-toolbar-buttons-panel
                 get-interactions-text)

        ; --------------------------------------------------------------------------------------------
        ;    0: some fields
        ; --------------------------------------------------------------------------------------------
        (define drracket-menu-bar (send this get-menu-bar))
        (define drracket-frame this)
        (define client #f)

        (define/public (disconnect-client!)
          (set! client #f)
          (on-dodona-disconnect)
          (update-submission-button))

        (define/public (connect-client! token)
          (when client (disconnect-client!))
          (set! client (new dodona-client% [host DODONA-HOST] [token token]))
          (on-dodona-connect)
          (update-submission-button))

        ; --------------------------------------------------------------------------------------------
        ;    1: 'Dodona' menu on top of DrRacket frame
        ; --------------------------------------------------------------------------------------------
        (define dodona-menu (new menu% [label DODONA-MENU-LABEL] [parent drracket-menu-bar]))

        ; --------------------------------------------------------------------------------------------
        ;    2: API token (we add a menu-item in the dodona-menu)
        ; --------------------------------------------------------------------------------------------
        (define/public (fetch-saved-api-token)
          (if (file-exists? API-TOKEN-FILE-ABS-PATH)
              (call-with-input-file API-TOKEN-FILE-ABS-PATH (lambda (in)
                                                              (let ((token (read-line in)))
                                                                (if (eof-object? token) #f token))))
              #f))

        (define API-token-menu-item
          (new menu-item%
               [label API-TOKEN-MENU-ITEM-LABEL]
               [parent dodona-menu]
               [callback (lambda (menu-item control-event)
                           (let* ((frame (new frame%
                                              [label API-TOKEN-MENU-ITEM-LABEL]
                                              [parent drracket-frame]
                                              [border (* DEFAULT-BORDER 2)]
                                              [spacing (* DEFAULT-SPACING 3)]))
                                  (api-token-column (new vertical-pane%
                                                         [parent frame]
                                                         [alignment '(center center)]))
                                  (instructions-column (new vertical-pane%
                                                            [parent frame]
                                                            [alignment '(center center)]))
                                  (ok-button-row (new horizontal-pane%
                                                      [parent frame]
                                                      [alignment '(right center)]))
                                  (api-token-column-row (new horizontal-pane%
                                                             [parent api-token-column]
                                                             [alignment '(center center)]))
                                  (api-token-column-row2 (new horizontal-pane%
                                                              [parent api-token-column]
                                                              [alignment '(center center)]))
                                  (api-token-text-field (new text-field%
                                                             [label API-TOKEN-TEXT-FIELD-LABEL]
                                                             [parent api-token-column-row]
                                                             [init-value (let ((api-token (fetch-saved-api-token)))
                                                                           (if api-token api-token ""))]
                                                             [min-width API-TOKEN-TEXT-FIELD-MIN-WIDTH]))
                                  (status-message (new message%
                                                       [label ""]
                                                       [parent api-token-column-row2]
                                                       [style (list 'deleted)] ;initially not shown
                                                       [auto-resize #t]))
                                  (log-in-button (new button%
                                                      [label LOG-IN-BUTTON-LABEL]
                                                      [parent api-token-column-row]
                                                      [callback (lambda (button control-event)
                                                                  (log-in-button-callback
                                                                   button
                                                                   api-token-text-field
                                                                   status-message
                                                                   api-token-column-row2))]))

                                  (instructions (let* ((group-box (new group-box-panel%
                                                                       [label INSTRUCTIONS-GROUP-BOX-LABEL]
                                                                       [parent instructions-column]
                                                                       [spacing DEFAULT-SPACING]
                                                                       [alignment '(left center)]))
                                                       (first-row (new horizontal-pane% [parent group-box])))
                                                  (define (make-instruction-message parent msg)
                                                    (new message%
                                                         [label msg]
                                                         [parent parent]
                                                         [auto-resize #t]
                                                         [font BIG-FONT]))

                                                  ;in first-row:
                                                  (make-instruction-message first-row INSTRUCTION-STEP1-MSG-LABEL)
                                                  (new button%
                                                       [label GO-TO-DODONA-BUTTON-LABEL]
                                                       [parent first-row]
                                                       [font BIG-FONT]
                                                       [callback (lambda (button control-event)
                                                                   (send-url DODONA-PROFILE-URL))])
                                                  ;below first-row:
                                                  (make-instruction-message group-box INSTRUCTION-STEP2-MSG-LABEL)
                                                  (make-instruction-message group-box INSTRUCTION-STEP3-MSG-LABEL)))

                                  (ok-button (new button%
                                                  [label OK-BUTTON-LABEL]
                                                  [parent ok-button-row]
                                                  [callback (lambda (button control-event)
                                                              (send frame show #f))])))
                             (send frame center)
                             (send frame show #t)))]))

        (define/private (log-in-button-callback button text-field status-text status-text-parent)
          (define (write-to-file str)
            (call-with-output-file
                API-TOKEN-FILE-ABS-PATH
              (lambda (out) (display str out)) #:exists 'replace))

          ; Give the user some indication something is happening.
          (define (start-communicating-message initial-text)
            (thread (lambda ()
                      (send status-text set-label initial-text)
                      (let loop ((init-label initial-text)
                                 (curr-label initial-text)
                                 (count 0))
                        (sleep 0.5)
                        (send status-text set-label curr-label)
                        (cond ((thread-try-receive))
                              ((= count 3) (loop init-label init-label 0))
                              (else (loop init-label (string-append curr-label ".") (+ count 1))))))))

          (define (stop-communicating-loop ref)
            (thread-send ref 'stop)
            (thread-wait ref))

          (let ((api-token (send text-field get-value))
                (loop-ref (start-communicating-message COMMUNICATING-WITH-DODONA-MSG-LABEL)))
            (send button enable #f)
            (unless (send status-text is-shown?) (send status-text-parent add-child status-text))

            (thread (lambda ()
                      (with-handlers ([exn:fail:network:errno? (lambda (network-error)
                                                                 (disconnect-client!)
                                                                 (write-to-file "")
                                                                 (stop-communicating-loop loop-ref)
                                                                 (send status-text set-label NO-CONNECTION-MSG-LABEL))]
                                      [exn:fail? (lambda (exception)
                                                   (disconnect-client!)
                                                   (write-to-file "")
                                                   (stop-communicating-loop loop-ref)
                                                   (send status-text set-label NEW-API-TOKEN-NOT-VALID-MSG-LABEL))])
                        ;try to connect with api-token, if this fails, an exception is raised
                        (connect-client! api-token)
                        (write-to-file api-token)
                        (stop-communicating-loop loop-ref)
                        (send status-text set-label NEW-API-TOKEN-VALID-MSG-LABEL))
                      (send button enable #t)))))

        (new separator-menu-item% [parent dodona-menu]) ;separator to make UI more beautiful

        ; --------------------------------------------------------------------------------------------
        ;    3A: submenu used to select a course and a series (or WPO) (hierarchical)
        ; --------------------------------------------------------------------------------------------
        (define select-series-submenu (new menu%
                                           [label SELECT-SERIES-MENU-ITEM-LABEL]
                                           [parent dodona-menu]))

        ;hashes
        (define course-name->id (make-hash '()))   ;key: course-name,   val: course ID
        (define series-name->id (make-hash '()))   ;key: series-name,   val: series ID
        (define exercise-name->id (make-hash '())) ;key: exercise-name, val: exercise ID

        ;hashes are filled at initialization of UI elements
        ;these hashes are useful because we show names in the UI. We can fetch a name from the UI, and use a hash to retrieve its ID

        ;course submenus
        (define/public (make-course-submenu course-label)
          (new menu% [label course-label] [parent select-series-submenu]))

        ;series-menu-items (such as "Series 1: Basic Scheme")
        (define selected-series-menu-item #f) ;efficiently tracks selected series

        (define (set-selected-series-menu-item! new-val)
          (let ((old-val selected-series-menu-item))
            (set! selected-series-menu-item new-val)
            ;Update the submission-button, user might be able to submit after selecting series
            (when (not old-val) (update-submission-button))))

        (define/public (get-selected-course-submenu)
          (and selected-series-menu-item (send selected-series-menu-item get-parent)))

        (define/private (make-series-checkable-menu-item series-label course-submenu)
          (new checkable-menu-item% [label series-label] [parent course-submenu] [callback series-menu-item-callback]))

        (define (series-menu-item-callback checkable-menu-item control-event)
          (when selected-series-menu-item
            (send selected-series-menu-item check #f))

          (set-selected-series-menu-item! checkable-menu-item)
          (send checkable-menu-item check #t)

          ;update exercise-choice for the freshly selected series
          (send exercise-choice clear)
          (set! exercise-name->id (make-hash '()))
          (for-each (lambda (exercise)
                      (hash-set! exercise-name->id (send exercise get-name) (send exercise get-id))
                      (send exercise-choice append (send exercise get-name)))
                    (let* ((series-id (hash-ref series-name->id (send checkable-menu-item get-label)))
                           (exercise-manager (get-field exercise-manager client))
                           (series-manager (get-field series-manager client))
                           (series (send series-manager get series-id)))
                      (send exercise-manager get-all series))))

        ; --------------------------------------------------------------------------------------------
        ;    3B: choices in button-panel that shows exercises of chosen series
        ; --------------------------------------------------------------------------------------------
        (define exercise-choice (new choice%
                                     [label ""]
                                     [choices '()]
                                     [min-width EXERCISE-CHOICE-MIN-WIDTH]
                                     [parent (get-button-panel)]))

        ; --------------------------------------------------------------------------------------------
        ;    4: on-dodona-connect: initialize courses & course-series
        ;       on-dodona-disconnect: tear-down initialization courses & course-series
        ; --------------------------------------------------------------------------------------------
        (define/private (get-user)
          (and client ;return #f if client is #f
               (let ((root-response (send client get-root-response)))
                 (send root-response get-user))))

        (define/private (on-dodona-connect)
          (let* ((user (get-user))
                 (user-subscribed-courses (send user get-subscribed-courses))
                 (series-manager (get-field series-manager client)))
            (for-each (lambda (subbed-course)
                        (let* ((course-name (string-append (send subbed-course get-name)
                                                           (course-name-suffix subbed-course user-subscribed-courses)))
                               (submenu (make-course-submenu course-name)))
                          (hash-set! course-name->id
                                     course-name
                                     (send subbed-course get-id))
                          (for-each (lambda (series)
                                      (let* ((series-name (send series get-name))
                                             (menu-item (make-series-checkable-menu-item series-name submenu))
                                             (id (send series get-id)))
                                        (hash-set! series-name->id series-name id)))
                                    (send series-manager get-all subbed-course))))
                      user-subscribed-courses)))

        (define (course-name-suffix course courses)
          (define (shares-name? subbed-course courses)
            (findf (lambda (different-course)
                     (equal? (send subbed-course get-name)
                             (send different-course get-name)))
                   (remove subbed-course courses (lambda (course1 course2)
                                                   (= (send course1 get-id)
                                                      (send course2 get-id))))))
          (if (shares-name? course courses)
              (string-append " (" (send course get-year) ")")
              ""))

        (define/private (on-dodona-disconnect)
          (set! course-name->id (make-hash '()))
          (set! series-name->id (make-hash '()))
          (set! exercise-name->id (make-hash '()))

          (for-each (lambda (course-submenu)
                      (for-each (lambda (series-menu-item) (send series-menu-item delete))
                                (send course-submenu get-items))
                      (send course-submenu delete))
                    (send select-series-submenu get-items))

          ; Unselect course / series
          (set-selected-series-menu-item! #f)
          (send exercise-choice clear))

        ; --------------------------------------------------------------------------------------------
        ;    5: Button Management procedures
        ; --------------------------------------------------------------------------------------------
        (define (change-button-panel lambda)
          (send (get-button-panel) change-children lambda))

        (define/public (show-button btn remover)
          (remover)
          (change-button-panel (lambda (l) (cons btn l)))
          (sort-toolbar-buttons-panel))

        ; --------------------------------------------------------------------------------------------
        ;    6A: Submission buttons: submit-btn, submit-selection-button, submitting-button, cant-submit-button
        ;        Only one of these submission buttons is shown at a time.
        ; --------------------------------------------------------------------------------------------
        (define submitting? #f)
        (define selected-start-pos #f) ;for submit-selection-button: holds the start position of selected text in DrRacket code editor
        (define selected-end-pos #f)   ;similarly, holds the end position of selected text

        (define currently-shown-submission-button #f)

        (define submit-button (new switchable-button%
                                   [label SUBMIT-BUTTON-LABEL]
                                   [callback (lambda (button)
                                               (send this submit-button-callback))]
                                   [parent (get-button-panel)]
                                   [bitmap SUBMIT-BITMAP]))

        (define submit-selection-button (new switchable-button%
                                             [label "Submit Selection"]
                                             [callback (lambda (button)
                                                         (send this submit-button-callback))]
                                             [parent (get-button-panel)]
                                             [bitmap SUBMIT-BITMAP]))

        (define cant-submit-button (new switchable-button%
                                        [label CANNOT-SUBMIT-BUTTON-LABEL]
                                        [callback (lambda (button)
                                                    (send this submit-button-callback))] ;since this is the cant-submit-button, submit-button-callback will show an error-frame
                                        (parent (get-button-panel))
                                        (bitmap QUESTION-MARK-BITMAP)))

        (define submitting-button (new switchable-button%
                                       [label SUBMITTING-BUTTON-LABEL]
                                       [callback (lambda (button)
                                                   (when submitting? ;holds a submission object
                                                     (let ((url (send submitting? get-url)))
                                                       (send-url (substring url 0 (- (string-length url) 5))))))] ;remove .json, and open in default browser with send-url
                                       [parent (get-button-panel)]
                                       [bitmap SUBMITTING-BITMAP]))

        ;Remember: only one of the submission buttons is shown at a time
        (register-toolbar-buttons (list submit-button submit-selection-button cant-submit-button submitting-button)
                                  #:numbers (list 11 12 13 14)) ;the numbers indicate that the buttons should be at the left

        (define (remove-submit-buttons-from-panel)
          (change-button-panel (lambda (l) (remq submitting-button (remq cant-submit-button (remq submit-selection-button (remq submit-button l)))))))

        (remove-submit-buttons-from-panel) ;remove all buttons for now

        (define/public (show-submission-button submission-button)
          (when (not (eq? submission-button currently-shown-submission-button)) ;is the btn is already being shown?
            (set! currently-shown-submission-button submission-button)
            (show-button submission-button remove-submit-buttons-from-panel)))

        (define/public (show-submit-button)
          (show-submission-button submit-button))
        (define/public (show-submit-selection-button)
          (show-submission-button submit-selection-button))
        (define/public (show-submitting-button)
          (show-submission-button submitting-button))
        (define/public (show-cant-submit-button)
          (show-submission-button cant-submit-button))

        (define/public (update-submission-button [tag 'none])  ;shows one of the submission buttons. Method is called when the shown button must change
          (with-handlers ([exn:fail? (lambda (exception) #f)])
            (cond
              (submitting? void) ;do nothing if submitting
              ((or (not client)
                   (not selected-series-menu-item)) (show-cant-submit-button))
              ((eq? tag 'selection-changed) (let* ((defs-text (get-definitions-text))
                                                   (start-pos (send defs-text get-start-position))
                                                   (end-pos (send defs-text get-end-position)))
                                              (cond ((= start-pos end-pos) ;true when no text is currently selected in the DrRacket code editor
                                                     (set! selected-start-pos #f)
                                                     (set! selected-end-pos #f)
                                                     (show-submit-button))
                                                    (else (set! selected-start-pos start-pos)
                                                          (set! selected-end-pos end-pos)
                                                          (show-submit-selection-button)))))
              (else (show-submit-button)))))

        (update-submission-button) ;call it to initialize

        ; --------------------------------------------------------------------------------------------
        ;    6B: Submit button callback
        ;
        ;    submit-button-callback starts a cascade of method calls to submit an exercise. Here are the methods and responsibilities:
        ;
        ;    step 1) submit-button-callback: entry-point for a submission. Retrieves code to submit, and also information about the course, series, exercise
        ;    step 2) submit-exercise: an initial submission to Dodona. Dodona responds back, but we must still wait for it to execute tests
        ;    step 3) refetch submission: refetch the submission until we get a full response from Dodona (including test results)
        ;    step 4) present submission: present the submission to the user
        ; --------------------------------------------------------------------------------------------

        (define/private (failsafe exception-handler) ;exception-handler = 0-arg lambda
          (with-handlers ([exn:fail? (lambda (exception)
                                       (let* ((frame (new frame%
                                                          [label GENERAL-ERROR-TITLE-LABEL]
                                                          [alignment '(center center)]
                                                          [min-width 500]
                                                          [min-height 100]))
                                              (msg (new message% [parent frame]
                                                        [label GENERAL-ERROR-LABEL])))
                                         (send frame show #t)))])
            (exception-handler))
          (set! submitting? #f)
          (update-submission-button))

        (define/public (submit-button-callback)
          (cond
            ((not client) (draw-submission-error-frame NO-VALID-API-TOKEN-FRAME-LABEL NO-VALID-API-TOKEN-MSG-LABEL))
            ((not selected-series-menu-item) (draw-submission-error-frame NO-SELECTED-SERIES-FRAME-LABEL NO-SELECTED-SERIES-MSG-LABEL))
            (else
             (let* ((code (if selected-start-pos
                              (send (get-definitions-text) get-text selected-start-pos selected-end-pos) ;only submit the selected code
                              (send (get-definitions-text) get-text)))
                    (name (send exercise-choice get-string-selection))
                    (course-name (send (get-selected-course-submenu) get-label)) ;twee keer Structuur van Computerprogramma's 1
                    (course-ID (hash-ref course-name->id course-name))
                    (series-name (send selected-series-menu-item get-label))
                    (series-ID (hash-ref series-name->id series-name))
                    (exercise-name (send exercise-choice get-string-selection))
                    (exercise-ID (hash-ref exercise-name->id exercise-name)))
               (submit-exercise course-ID series-ID exercise-ID code)))))

        (define/private (submit-exercise courseID seriesID exerciseID code)
          (define (draw-no-connection-frame)
            (draw-submission-error-frame NO-CONNECTION-FRAME-LABEL
                                         NO-CONNECTION-MSG-LABEL))
          (with-handlers ([exn:fail:network:errno? (lambda (network-error)
                                                     (failsafe (lambda ()
                                                                 (draw-no-connection-frame))))])
            (let* ((submission-manager (get-field submission-manager client))
                   (submission-response (send submission-manager submit courseID seriesID exerciseID code)) ;not a complete response:
                   (submission-ID (send submission-response get-id))                                        ;we must still wait for the tests to be ran by dodona
                   (submission (send submission-manager get submission-ID)))
              (set! submitting? submission)  ;submitting
              (show-submitting-button)       ;show corresponding button
              (thread (lambda () (with-handlers ([exn:fail:network:errno? (lambda (network-error)
                                                                            (failsafe (lambda ()
                                                                                        (draw-no-connection-frame))))])
                                   (refetch-submission submission submission-ID submission-manager #t)))))))

        (define/private (refetch-submission submission submission-ID submission-manager first-call?)
          (if first-call?
              (sleep 2) ;sleep to give time for Dodona to run tests
              (sleep 0.5))

          (let ((status (send submission get-status)))
            (if (or (equal? status "running")
                    (equal? status "queued")) ;if submission is running or queued, then refetch it
                (refetch-submission (send submission-manager get submission-ID) submission-ID submission-manager #f)
                (submission-completed submission status))))

        (define/private (submission-completed submission status)
          (set! submitting? #f)       ;submission completed
          (update-submission-button)  ;so we must update submission-button (from submitting to can-submit)
          (dispatch-submission submission status))

        (define/private (dispatch-submission submission status)
          (cond
            ((or (equal? status "correct")
                 (equal? status "wrong"))
             (present-submission submission))
            ((equal? status "runtime error")
             (present-runtime-error-submission submission))
            (else (present-exceptional-submission submission))))

        ;Presentation functions:
        (define/public (make-submission-frame)
          (new frame%
               [label SUBMISSION-FRAME-LABEL]
               [border DEFAULT-BORDER]
               [spacing DEFAULT-SPACING]
               [min-height 0]
               [min-width 0]))

        (define/public (make-ok-button parent frame)
          (new button%
               [parent parent]
               [label OK-BUTTON-LABEL]
               [callback (lambda (button control-event)
                           (send frame show #f))]))

        (define/public (make-open-in-dodona-button parent submission)
          (new button% ;;;open in dodona button
               [parent parent]
               [label OPEN-IN-DODONA-BUTTON-LABEL]
               [callback (lambda (button control-event)
                           (let ((submission-url (send submission get-url))) ;a string of the url
                             (send-url (substring submission-url 0 (- (string-length submission-url) 5)))))]))

        (define/public (make-code-text-field code parent min-width min-height)
          (let ((text-field (new text-field%
                                 [label ""]
                                 [parent parent]
                                 [min-width min-width]
                                 [min-height min-height]
                                 [font CODE-FONT]
                                 [style (list 'multiple)]
                                 [enabled false])))
            (send text-field set-field-background (send the-color-database find-color "dim gray"))
            (send text-field set-value code)))

        (define (make-code-text-field-or-message output-msg font parent)
          (if (or (is-a? output-msg bitmap%)
                  (label-string? output-msg))
              (new message%
                   [parent parent]
                   [font font]
                   [auto-resize #t]
                   [label output-msg])
              (make-code-text-field output-msg parent 200 200)))

        (define/public (general-tab-panel-callback tab-panel control-event submission test-tabs test-tab-handler)
          ;remove children
          (for-each (lambda (child)
                      (send tab-panel delete-child child))
                    (send tab-panel get-children))
          ;add content for the selected-tab
          (let* ((selected-tab-label (send tab-panel get-item-label (send tab-panel get-selection)))
                 (selected-tab (findf (lambda (tab)
                                        (equal? (send tab get-description)
                                                selected-tab-label)) test-tabs))
                 (code-tab? (equal? selected-tab-label "Code")))
            (if code-tab?
                (let ((code (send submission get-code)))
                  (make-code-text-field code tab-panel 400 400))
                (test-tab-handler selected-tab))))

        (define/public (general-present-submission submission presenter)
          (let* ((frame (make-submission-frame))
                 (submission-accepted? (send submission get-accepted?))
                 (correctness-msg (new message%
                                       [parent frame]
                                       [label (if submission-accepted?
                                                  LARGE-CORRECT-CHECK-MARK-BITMAP
                                                  LARGE-WRONG-CHECK-MARK-BITMAP)]
                                       [font normal-control-font]
                                       [auto-resize #t]))
                 (summary-msg (new message%
                                   [parent frame]
                                   [label (let ((summary (send submission get-summary)))
                                            (if (eq? 'null summary)
                                                NULL-MSG-LABEL
                                                summary))]
                                   [font normal-control-font]
                                   [auto-resize #t])))
            (presenter frame)))

        (define/public (format-table-content lst)
          (define (group-by-4 flat-lst) ;flat-lst should be a lst whose length is divisble by 4
            (if (null? flat-lst)
                '()
                (cons (list (first flat-lst)
                            (second flat-lst)
                            (third flat-lst)
                            (fourth flat-lst))
                      (group-by-4 (cddddr flat-lst)))))
          (group-by-4 (flatten lst)))

        (define/public (present-submission submission) ;function that presents submission to user (including tests)
          (general-present-submission submission
                                      (lambda (frame)
                                        (let* ((submission-accepted? (send submission get-accepted?))
                                               (submission-test (send submission get-submission-test))
                                               (test-tabs (send submission-test get-test-tabs))) ;list of test-tab% objects (which is a dodona resource))  

                                          (let* ((tab-panel-callback
                                                  (lambda (tab-panel control-event)
                                                    (general-tab-panel-callback tab-panel
                                                                                control-event
                                                                                submission
                                                                                test-tabs
                                                                                (lambda (selected-tab)
                                                                                  (let* ((table (new table-panel%
                                                                                                     [parent tab-panel]
                                                                                                     [max-height 500]
                                                                                                     [dimensions (list 36 4)] ;table with 15 rows, and 4 columns. This is only okay because column-spacing is small (< 5)
                                                                                                     [row-spacing 50]
                                                                                                     [column-spacing 3]))
                                                                                         (make-table-message (lambda (label font)
                                                                                                               (make-code-text-field-or-message label font table))))

                                                                                    (make-table-message TEST-CODE-MSG-LABEL       BIG-FONT)
                                                                                    (make-table-message EXPECTED-RESULT-MSG-LABEL BIG-FONT)
                                                                                    (make-table-message CORRECT?-MSG-LABEL        BIG-FONT)
                                                                                    (make-table-message ""                        normal-control-font)

                                                                                    (make-table-message "" normal-control-font)
                                                                                    (make-table-message "" normal-control-font)
                                                                                    (make-table-message "" normal-control-font)
                                                                                    (make-table-message "" normal-control-font)
                                                                            
                                                                                    (for-each (lambda (test-group)

                                                                                                (make-table-message "" normal-control-font)
                                                                                                (make-table-message "" normal-control-font)
                                                                                                (make-table-message "" normal-control-font)
                                                                                                (make-table-message "" normal-control-font)

                                                                                                (make-table-message (send test-group get-description) BIG-FONT)
                                                                                                (make-table-message "" normal-control-font)
                                                                                                (make-table-message "" normal-control-font)
                                                                                                (make-table-message "" normal-control-font)
                                                                                        
                                                                                                (for-each (lambda (test-case)
                                                                                                            (let ((test-case-description (send test-case get-description)))
                                                                                                              (for-each (lambda (test-case-test)
                                                                                                                          (let ((test-code (send test-case-test get-test-code))
                                                                                                                                (expected-val (send test-case-test get-expected))
                                                                                                                                (generated-val (send test-case-test get-generated)))

                                                                                                                            (make-table-message test-code CODE-FONT)
                                                                                                                            (make-table-message expected-val CODE-FONT)

                                                                                                                            (make-table-message (if (or (equal? expected-val generated-val)
                                                                                                                                                        (with-handlers ([exn:fail? (lambda (e) #f)])
                                                                                                                                                          (= (string->number expected-val)
                                                                                                                                                             (string->number generated-val))))
                                                                                                                                                    CORRECT-CHECK-MARK-BITMAP
                                                                                                                                                    WRONG-CHECK-MARK-BITMAP)
                                                                                                                                                normal-control-font)

                                                                                                                            (new button% ;COPY to REPL button
                                                                                                                                 [parent table]
                                                                                                                                 [label COPY-TO-REPL-BUTTON-LABEL]
                                                                                                                                 [callback (lambda (button control-event)
                                                                                                                                             (let* ((interactions-text (get-interactions-text))
                                                                                                                                                    (last-position (send interactions-text last-position)))
                                                                                                                                               (send frame show #f)
                                                                                                                                               (send interactions-text begin-edit-sequence)
                                                                                                                                               (send interactions-text insert test-code last-position)
                                                                                                                                               (send interactions-text end-edit-sequence)))])))
                                                                                                                        (send test-case get-test-case-tests))))
                                                                                                          (send test-group get-test-cases)))
                                                                                              (send selected-tab get-test-groups)))

                                                                          
                                                                          

                                                                                  (define-values (w h) (send tab-panel get-graphical-min-size))
                                                                                  (send frame resize 500 800) ;resize the frame
                                                                                  ))))

                                                 (tab-panel (new tab-panel% ;tab-panel for test-tabs and code-tab
                                                                 [parent frame]
                                                                 [choices (append (map (lambda (test-tab)
                                                                                         (send test-tab get-description))
                                                                                       test-tabs)
                                                                                  (list "Code"))]
                                                                 [callback tab-panel-callback]
                                                                 [spacing DEFAULT-SPACING]
                                                                 [border DEFAULT-BORDER])))
                                            (tab-panel-callback tab-panel void))

                                          (let ((row (new horizontal-pane%
                                                          [parent frame]
                                                          [alignment '(center center)])))

                                            (make-open-in-dodona-button row submission)
                                            (make-ok-button row frame))

                                          (send frame center)
                                          (send frame show #t)

                                          (set-last-submission! (cons frame submission-accepted?)) ;provides information for last-submission-button (see below)
                                          ))))

        (define/public (present-runtime-error-submission submission) ;function that presents an exceptional submission to user. For example one with a runtime error
          (general-present-submission submission
                                      (lambda (frame)   
                                        (let* ((submission-accepted? (send submission get-accepted?))
                                               (submission-test (send submission get-submission-test))
                                               (test-tabs (send submission-test get-test-tabs))
                                               (output-tab (car (send submission-test get-test-tabs)))
                                               (output-msg (send output-tab get-scheme-error))
                                               (exception-tab-panel-callback
                                                (lambda (tab-panel control-event)
                                                  (general-tab-panel-callback tab-panel
                                                                              control-event
                                                                              submission
                                                                              test-tabs
                                                                              (lambda (selected-tab)
                                                                                (make-code-text-field-or-message output-msg CODE-FONT tab-panel)
                                                                                (define-values (l r) (send tab-panel get-graphical-min-size))
                                                                                (send frame resize l r))))) ;resize the frame
                                               (exception-tab-panel (new tab-panel%
                                                                         [parent frame]
                                                                         [choices (append (map (lambda (test-tab)
                                                                                                 (send test-tab get-description))
                                                                                               test-tabs)
                                                                                          (list "Code"))]
                                                                         [callback exception-tab-panel-callback]
                                                                         [spacing DEFAULT-SPACING]
                                                                         [border DEFAULT-BORDER])))
            
                                          (exception-tab-panel-callback exception-tab-panel #f)

                                          (let ((row (new horizontal-pane%
                                                          [parent frame]
                                                          [alignment '(center center)])))

                                            (make-open-in-dodona-button row submission)
                                            (make-ok-button row frame))

                                          (send frame center)
                                          (send frame show #t)

                                          (set-last-submission! (cons frame submission-accepted?)))))) ;provides information for last-submission-button (see below)))

        (define/public (present-exceptional-submission submission)
          (general-present-submission submission
                                      (lambda (frame)
                                        (let ((submission-accepted? (send submission get-accepted?))
                                              (row (new horizontal-pane%
                                                        [parent frame]
                                                        [alignment '(center center)])))

                                          (make-open-in-dodona-button row submission)
                                          (make-ok-button row frame)

                                          (send frame center)
                                          (send frame show #t)

                                          (set-last-submission! (cons frame submission-accepted?))))))
                                        
          

        ; --------------------------------------------------------------------------------------------
        ;    7: Last Submission button
        ; --------------------------------------------------------------------------------------------
        (define last-submission-frame #f) ;holds the frame% shown for the last submission

        (define/public (set-last-submission! pair)
          (let ((frame (car pair))
                (submission-accepted? (cdr pair)))
            (set! last-submission-frame frame)
            (if submission-accepted?
                (show-last-submission-correct-button)
                (show-last-submission-wrong-button))))

        (define last-submission-button-callback
          (lambda (button)
            (and last-submission-frame
                 (send last-submission-frame show #t))))

        ;last-submission-correct-button or last-submission-wrong-button is shown at a time
        (define last-submission-correct-button (new switchable-button%
                                                    [label LAST-SUBMISSION-BUTTON-LABEL]
                                                    [callback last-submission-button-callback]
                                                    [parent (get-button-panel)]
                                                    [bitmap CORRECT-CHECK-MARK-BITMAP]))

        (define last-submission-wrong-button (new switchable-button%
                                                  [label LAST-SUBMISSION-BUTTON-LABEL]
                                                  [callback last-submission-button-callback]
                                                  [parent (get-button-panel)]
                                                  [bitmap WRONG-CHECK-MARK-BITMAP]))

        (register-toolbar-buttons (list last-submission-correct-button last-submission-wrong-button) #:numbers (list 18 19))

        (define (remove-last-submission-buttons-from-panel)
          (change-button-panel (lambda (l) (remq last-submission-wrong-button (remq last-submission-correct-button l)))))
        (remove-last-submission-buttons-from-panel) ;initially removed

        (define/public (show-last-submission-button last-submission-button)
          (show-button last-submission-button remove-last-submission-buttons-from-panel))

        (define/public (show-last-submission-correct-button)
          (show-last-submission-button last-submission-correct-button))

        (define/public (show-last-submission-wrong-button)
          (show-last-submission-button last-submission-wrong-button))
        ; --------------------------------------------------------------------------------------------
        ;    8: initialize client by using the saved token (if any)
        ; --------------------------------------------------------------------------------------------
        (with-handlers ([exn:fail? (lambda (e) (displayln e))])
          (let ((api-token (fetch-saved-api-token)))
            (and api-token
                 (connect-client! api-token))))

        ; --------------------------------------------------------------------------------------------
        ;    A utility: draw-info-frame and draw-error-frame
        ; --------------------------------------------------------------------------------------------
        (define/public (draw-info-frame frame-label msg-label)
          (let* ((frame (new frame%
                             [label frame-label]
                             [border DEFAULT-BORDER]
                             [spacing DEFAULT-SPACING]
                             [min-width 400]
                             [min-height 150]))
                 (msg (new message%
                           [label msg-label]
                           [parent frame]
                           [auto-resize #t]))
                 (row [new horizontal-pane%
                           [parent frame]
                           [alignment '(right center)]]))
            (new button%
                 [label OK-BUTTON-LABEL]
                 [parent row]
                 [callback (lambda (button control-event)
                             (send frame show #f))])
            (send frame center)
            (send frame show #t)))

        (define/public (draw-submission-error-frame frame-label msg-label)
          (draw-info-frame frame-label msg-label))))

    (define definitions-text-mixin
      (mixin (drracket:unit:definitions-text<%> (class->interface text%)) ()
        (super-new)
        (inherit get-tab)

        (define/public (get-drracket-frame)
          (send (get-tab) get-frame))

        (define/augment (after-set-next-settings lang-settings) ;this method is called by DrRacket after the language settings change
          (send (get-drracket-frame) update-submission-button))

        (define/augment (after-set-position)                    ;this method is called by DrRacket after the selected text in the code editor changes
          (send (get-drracket-frame) update-submission-button 'selection-changed))
        ))

    (define (phase1) (void))
    (define (phase2) (void))

    (drracket:get/extend:extend-definitions-text definitions-text-mixin)
    (drracket:get/extend:extend-unit-frame frame-mixin)))

