#lang racket
(require rackunit)
(require "../../../dodona-api/dodona-client.rkt")
(require "../../../dodona-api/resources/exercise.rkt")
(require "../resource-validators.rkt")
(require "../constants.rkt")

(test-case "SUBMISSION MANAGER TEST"
           (define client (new dodona-client% [host DODONA-TEST-URL] [token "zeus"]))

           (define submission-manager (get-field submission-manager client))
           (define a-submission-id #f)
           (define exercise-stub (new exercise% [exercise-data (make-hasheq (list (cons 'id 128487518)))])) ;exercise for submission

           (let* ((courseID 3)
                  (seriesID 23)
                  (exerciseID 128487518)
                  (first-post-response (send submission-manager submit courseID seriesID exerciseID "float fac(float fac) {...}")))

             (sleep 5) ;sleep is needed since submit frequency is limited

             (let* ((exerciseID2 805381696)
                    (second-post-response (send submission-manager submit-exercise exerciseID2 "(define (sign another) another)")))

               (validate-submission-created-response first-post-response) ;validate submission-created responses
               (validate-submission-created-response second-post-response)

               (set! a-submission-id (send first-post-response get-id))
               ))

           (define fetched-submission (send submission-manager get a-submission-id))
           (validate-submission fetched-submission)

           (define user-submissions-info (send submission-manager get-user-submissions-info))

           (for-each validate-submission-info user-submissions-info)

           (define user-exercise-submissions (send submission-manager get-exercise-submissions-info exercise-stub))
           (for-each validate-submission-info user-exercise-submissions)

           (define first-user-submission (send (car user-submissions-info) get-exercise-url))
           (define first-user-exercise-submission (send (car user-exercise-submissions) get-exercise-url))

           (define (get-exercise-urls submissions-info)
             (map (lambda (info)
                    (send info get-exercise-url))
                  submissions-info))

           (check-false (andmap (lambda (exe-url) ;check that all submissions dont have the same exercise-url
                                 (equal? exe-url first-user-submission))
                               (get-exercise-urls user-submissions-info)))

           (check-true (andmap (lambda (exe-url) ;check that all submissions do have the same exercise-url
                                 (equal? exe-url first-user-exercise-submission))
                               (get-exercise-urls user-exercise-submissions))))
