#lang racket
(require rackunit)
(require "../../dodona-api/responses/root-response.rkt")
(require "../../dodona-api/resources/user.rkt")
(require "../../dodona-api/resources/submission.rkt")
(require "../../dodona-api/resources/submission-info.rkt")
(require "../../dodona-api/resources/series.rkt")
(require "../../dodona-api/resources/exercise.rkt")
(require "../../dodona-api/resources/course.rkt")
(require "../../dodona-api/responses/submission-created-response.rkt")

;;A validator checks the fields of a dodona-fetched resource, for example a user.

(provide (all-defined-out))

(define (validate-root-response root-response)
  (check-true (is-a? root-response root-response%))
  (check-true (list? (send root-response get-deadline-series)))
  (validate-user (send root-response get-user)))

(define (validate-user user)
  (check-true (is-a? user user%))
  (check-true (string? (send user get-email)))
  (check-true (number? (send user get-correct-exercises)))
  (check-true (string? (send user get-first-name)))
  (check-true (number? (send user get-id)))
  (check-true (string? (send user get-last-name)))
  (check-true (number? (send user get-submission-count)))
  (check-true (string? (send user get-submissions-url)))
  (check-true (list? (send user get-subscribed-courses)))
  (check-true (string? (send user get-username))))

(define (validate-submission submission)
  (check-true (is-a? submission submission%))
  (check-true (boolean? (send submission get-accepted?)))
  (check-true (string? (send submission get-code)))
  (check-true (string? (send submission get-course-url)))
  (check-true (string? (send submission get-creation-timestamp)))
  (check-true (string? (send submission get-exercise-url)))
  (check-true (number? (send submission get-id)))
  (check-true (string? (send submission get-status)))
  (check-true (or (equal? 'null (send submission get-summary))
                  (string? (send submission get-summary))))
  (check-true (string? (send submission get-url)))
  (check-true (is-a? (send submission get-submission-test) object%)))

(define (validate-series series)
  (check-true (is-a? series series%))
  (check-true (or (equal? (send series get-deadline) 'null)
                  (string? (send series get-deadline))))
  (check-true (string? (send series get-description)))
  (check-true (string? (send series get-exercises-url)))
  (check-true (number? (send series get-id)))
  (check-true (string? (send series get-name)))
  (check-true (number? (send series get-order)))
  (check-true (string? (send series get-visibility)))
  (check-true (string? (send series get-url))))

(define (validate-exercise exercise)
  (check-true (is-a? exercise exercise%))
  (check-true (or (equal? 'null (send exercise get-boilerplate))
                  (string? (send exercise get-boilerplate))))
  (check-true (boolean? (send exercise get-has-correct-solution?)))
  (check-true (boolean? (send exercise get-has-solution?)))
  (check-true (number? (send exercise get-id)))
  (check-true (boolean? (send exercise get-last-solution-is-best?)))
  (check-true (string? (send exercise name)))
  (check-true (string? (send exercise get-url))))

(define (validate-course course)
  (check-true (is-a? course course%))
  (check-true (string? (send course get-color)))
  (check-true (number? (send course get-id)))
  (check-true (string? (send course get-name)))
  (check-true (string? (send course get-teacher)))
  (check-true (string? (send course get-series-url)))
  (check-true (string? (send course get-year)))
  (check-true (string? (send course get-url))))

(define (validate-submission-created-response submission-created-response)
  (check-true (is-a? submission-created-response submission-created-response%))
  (check-true (or (eq? (send submission-created-response get-course-id) 'null)
                  (number? (send submission-created-response get-course-id))))
  (check-true (number? (send submission-created-response get-exercise-id)))
  (check-true (number? (send submission-created-response get-id)))
  (check-true (string? (send submission-created-response get-status)))
  (check-true (string? (send submission-created-response get-url))))

(define (validate-submission-info submission-info)
  (check-true (is-a? submission-info submission-info%))
  (check-true (boolean? (send submission-info get-accepted?)))
  (check-true (string? (send submission-info get-creation-timestamp)))
  (check-true (string? (send submission-info get-exercise-url)))
  (check-true (number? (send submission-info get-id)))
  (check-true (string? (send submission-info get-status)))
  (check-true (string? (send submission-info get-url))))
